{
This file is part of Unreal Tournament System Config version 210f.

Copyright (C) 2000, 2003 Simon McVittie.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
}

unit BotFiles;

interface

uses TBigKegOBotsDefine, TBotDefine, SysUtils, Classes, Dialogs,Inifiles;

{procedure SaveBotList(BotList:TBigKegOBots;Filename:String);}
procedure LoadBotList(BotList:TBigKegOBots;Filename:String);
procedure LoadV2BotList(BotList:TBigkegoBots;Filename:string);
procedure SaveV2BotList(Botlist:TBigkegoBots;Filename:string);
procedure ImportBotList(BotList:TBigKegOBots;IniFile:String);
procedure ExportBotList(BotSquad:TBotsquad;IniFile:String);
function Pick32Bots(BotList:TBigKegOBots):TBotSquad;

implementation
uses UTSharedUtils1;

const hHeader='UT_Server_Config_BOTS_File';
      hPrefix='BotData';
      vNum='NumBots';
      vName='Name';
      vClass='Class';
      vSkin='Skin';
      vFace='Face';
      vSkill='Skill';
      vAccu='Accuracy';
      vCombatStyle='Aggression';
      vAlert='Alertness';
      vCamp='Camping';
      vStrafe='StrafingAbility';
      vFave='FaveWeapon';
      vVoice='Voicepack';
      vJumpy='JumpyBehaviour';
      vTeam='Team';

procedure LoadV2BotList(BotList:TBigKegOBots;Filename:String);
{deliberately doesn't clear the list - do so before calling if necessary}
var b:TBot; IniF: TIniFile; Num,i:integer;  s:string;
Begin
  IniF:=TIniFile.Create(FileName);
  Num:=IniF.ReadInteger(hHeader,vNum,0);
    if (Num>0) then
      for i:=1 to Num do begin
        s:=hPrefix+IntToStr(i);
        b.Name:=IniF.ReadString(s,vName,'Unnamed'+IntToStr(i));
        b.Team:=IniF.ReadInteger(s,vTeam,0) mod 256;
        b.Skill:=IniF.ReadInteger(s,vSkill,0)/100;
        b.Accuracy:=IniF.ReadInteger(s,vAccu,0)/100;
        b.CombatStyle:=IniF.ReadInteger(s,vCombatStyle,0)/100;
        b.Alertness:=IniF.ReadInteger(s,vAlert,0)/100;
        b.Camping:=IniF.ReadInteger(s,vCamp,0)/100;
        b.StrafingAbility:=IniF.ReadInteger(s,vStrafe,0)/100;
        b.FavoriteWeapon:=IniF.ReadString(s,vFave,'');
        b.ModelClass:=IniF.ReadString(s,vClass,'');
        b.Skin:=IniF.ReadString(s,vSkin,'');
        b.Face:=IniF.ReadString(s,vFace,'');
        b.Voice:=IniF.ReadString(s,vVoice,'');
        b.Jumpy:=IniF.ReadBool(s,vJumpy,FALSE);
        BotList.Append(b);
      end;{for i}
  IniF.Free;
End;

procedure SaveV2BotList(Botlist:TBigkegoBots;Filename:string);
var i:integer; s:string; IniF:TIniFile;
Begin
  IniF:=TIniFile.Create(FileName);
  IniF.WriteInteger(hHeader,vNum,Botlist.Count);
    if (Botlist.Count>0) then
      for i:=0 to Botlist.Count-1 do begin
        s:=hPrefix+IntToStr(i+1);
        IniF.WriteString(s,vName,Botlist[i].Name);
        IniF.WriteInteger(s,vTeam,Botlist[i].Team);
        IniF.WriteInteger(s,vSkill,Round(botlist[i].Skill*100));
        IniF.WriteInteger(s,vAccu,Round(botlist[i].Accuracy*100));
        IniF.WriteInteger(s,vCombatStyle,Round( botlist[i].CombatStyle*100));
        IniF.WriteInteger(s,vAlert,Round( botlist[i].Alertness*100));
        IniF.WriteInteger(s,vCamp,Round( botlist[i].Camping*100));
        IniF.WriteInteger(s,vStrafe,Round( botlist[i].StrafingAbility*100));
        IniF.WriteString(s,vFave,botlist[i].FavoriteWeapon);
        IniF.WriteString(s,vClass,botlist[i].ModelClass);
        IniF.WriteString(s,vSkin,botlist[i].Skin);
        IniF.WriteString(s,vFace,botlist[i].Face);
        IniF.WriteString(s,vVoice,botlist[i].Voice);
        IniF.WriteBool(s,vJumpy,botlist[i].Jumpy)
      end;{for i}
  IniF.Free;
End;

{procedure SaveBotList(BotList:TBigKegOBots;Filename:String);
var i:integer; B:TBot; F:file of TBot;
Begin
  assignFile(F,FileName);
  Rewrite(F);
  for i:=0 to BotList.Count-1 do
    begin
    B:=BotList.Data[i];
    Write(F,B);
    end;
  CloseFile(F);
End;          }

procedure LoadBotList(BotList:TBigKegOBots;Filename:String);
{deliberately doesn't clear the list - do so before calling if necessary}
var b:TBot; F: file of TBot;
Begin
  assignFile(F,FileName);
  Reset(F);
  while not eof(F) do
    begin
      read(F,b);
      BotList.Append(b);
    end;
  CloseFile(F);
End;

procedure ImportBotList(BotList:TBigKegOBots;IniFile:String);
var F:text;
    i:integer;
    Squad:TBotSquad;
    Quit,QuitCurrent:boolean;
    S:string;
const BotSection='[BOTPACK.CHALLENGEBOTINFO]';
      BotName='BOTNAMES[';
      BotHeaderEnd=']=';
      BotFace='BOTFACES[';
      BotSkin='BOTSKINS[';
      BotModel='BOTCLASSES[';
      BotTeam='BOTTEAMS[';
      BotVoice='VOICETYPE[';
      BotSkill='BOTSKILLS[';
      BotAccuracy='BOTACCURACY[';
      BotCombatStyle='COMBATSTYLE[';
      BotCamping='CAMPING[';
      BotStrafing='STRAFINGABILITY[';
      BotAlertness='ALERTNESS[';
      BotFave='FAVORITEWEAPON[';
      BotJumpy='BOTJUMPY[';
{BE CAREFUL. Infiltration has its own bot library which looks similar. This is
stored under InfiltrationUT.InfBotInfo. I might add support for this soon.}
Begin
for i:=0 to 31 do Squad[i]:=BlankBot;
Quit:=FALSE;
{can't use normal INI techniques because UT INIs can have multiple values for
a key, as used for ServerPackages}
{ i) Read through until you reach bot-info heading}
QuitCurrent:=FALSE;
assignFile(F,IniFile);
reset(F);
repeat
if eof(F) then Quit:=TRUE
else begin
  readln(F,S);
  if (uppercase(S)=BotSection) then QuitCurrent:=TRUE;
end;{if}
until Quit OR QuitCurrent;

QuitCurrent:=FALSE;
if not Quit then
{ ii) Get each line starting with the correct key and add it to the squad}
repeat
if eof(F) then QuitCurrent:=TRUE
else begin
  readln(F,S);

  if (S='') then QuitCurrent:=TRUE else
  begin
    i:=0;
    Quit:=FALSE;
    while (i<32) and (not Quit) do
    begin
    {begin big, messy if-elseif}
    {text}
    if (StartsWith(UpperCase(S),BotName+IntToStr(i)+BotHeaderEnd))
      then begin Quit:=TRUE; Squad[i].Name:=Copy(S,Pos('=',S)+1,255); end
    else if (StartsWith(UpperCase(S),BotFace+IntToStr(i)+BotHeaderEnd))
      then begin Quit:=TRUE; Squad[i].Face:=Copy(S,Pos('=',S)+1,255); end
    else if (StartsWith(UpperCase(S),BotSkin+IntToStr(i)+BotHeaderEnd))
      then begin Quit:=TRUE; Squad[i].Skin:=Copy(S,Pos('=',S)+1,255); end
    else if (StartsWith(UpperCase(S),BotModel+IntToStr(i)+BotHeaderEnd))
      then begin Quit:=TRUE; Squad[i].ModelClass:=Copy(S,Pos('=',S)+1,255); end
    else if (StartsWith(UpperCase(S),BotFave+IntToStr(i)+BotHeaderEnd))
      then begin Quit:=TRUE; Squad[i].FavoriteWeapon:=Copy(S,Pos('=',S)+1,255); end
    else if (StartsWith(UpperCase(S),BotVoice+IntToStr(i)+BotHeaderEnd))
      then begin Quit:=TRUE; Squad[i].Voice:=Copy(S,Pos('=',S)+1,255); end
    {float}
    else if (StartsWith(UpperCase(S),BotAccuracy+IntToStr(i)+BotHeaderEnd))
      then begin Quit:=TRUE; Squad[i].Accuracy:=FloatVal(Copy(S,Pos('=',S)+1,255)); end
    else if (StartsWith(UpperCase(S),BotCombatStyle+IntToStr(i)+BotHeaderEnd))
      then begin Quit:=TRUE; Squad[i].CombatStyle:=FloatVal(Copy(S,Pos('=',S)+1,255)); end
    else if (StartsWith(UpperCase(S),BotStrafing+IntToStr(i)+BotHeaderEnd))
      then begin Quit:=TRUE; Squad[i].StrafingAbility:=FloatVal(Copy(S,Pos('=',S)+1,255)); end
    else if (StartsWith(UpperCase(S),BotCamping+IntToStr(i)+BotHeaderEnd))
      then begin Quit:=TRUE; Squad[i].Camping:=FloatVal(Copy(S,Pos('=',S)+1,255)); end
    else if (StartsWith(UpperCase(S),BotSkill+IntToStr(i)+BotHeaderEnd))
      then begin Quit:=TRUE; Squad[i].Skill:=FloatVal(Copy(S,Pos('=',S)+1,255)); end
    else if (StartsWith(UpperCase(S),BotAlertness+IntToStr(i)+BotHeaderEnd))
      then begin Quit:=TRUE; Squad[i].Alertness:=FloatVal(Copy(S,Pos('=',S)+1,255)); end
    {byte}
    else if (StartsWith(UpperCase(S),BotTeam+IntToStr(i)+BotHeaderEnd))
      then begin Quit:=TRUE; Squad[i].Team:=ByteVal(Copy(S,Pos('=',S)+1,255)); end
    {boolean}
    else if (StartsWith(UpperCase(S),BotJumpy+IntToStr(i)+BotHeaderEnd))
      then begin Quit:=TRUE; Squad[i].Jumpy:=NOT (Copy(S,Pos('=',S)+1,1)='0');
      end;{that was horrible, wasn't it?}
    inc(i);
    end;{while}
  end;{else}
end;{else}
until QuitCurrent;
close(F);
for i:=0 to 31 do BotList.Append(Squad[i]);
End;

procedure ExportBotList(BotSquad:TBotSquad;IniFile:String);
var Buffer:TStringList;
    Done,Ready:boolean;
    F:text;
    S:string;
    i:integer;
const BotSection='[BOTPACK.CHALLENGEBOTINFO]';
      BotName='BOTNAMES[';
      BotHeaderEnd=']=';
      BotFace='BOTFACES[';
      BotSkin='BOTSKINS[';
      BotModel='BOTCLASSES[';
      BotTeam='BOTTEAMS[';
      BotVoice='VOICETYPE[';
      BotSkill='BOTSKILLS[';
      BotAccuracy='BOTACCURACY[';
      BotCombatStyle='COMBATSTYLE[';
      BotCamping='CAMPING[';
      BotStrafing='STRAFINGABILITY[';
      BotAlertness='ALERTNESS[';
      BotFave='FAVORITEWEAPON[';
      BotJumpy='BOTJUMPY[';
      wBotSection='[BotPack.ChallengeBotInfo]';
      wBotName='BotNames[';
      wBotFace='BotFaces[';
      wBotSkin='BotSkins[';
      wBotModel='BotClasses[';
      wBotTeam='BotTeams[';
      wBotVoice='VoiceType[';
      wBotSkill='BotSkills[';
      wBotAccuracy='BotAccuracy[';
      wBotCombatStyle='CombatStyle[';
      wBotCamping='Camping[';
      wBotStrafing='StrafingAbility[';
      wBotAlertness='Alertness[';
      wBotFave='FavoriteWeapon[';
      wBotJumpy='BotJumpy[';
Begin
Buffer:=TStringList.Create;
try

Done:=FALSE;
Ready:=FALSE;

AssignFile(F,IniFile);
Reset(F);

while not EOF(F) do
  begin
    readln(F,S);
    if (UpperCase(S)=BotSection) and (not Done) then Ready:=TRUE
      else if Ready AND (S='')
             then begin
               {write bot details before it...}
                 for i:=0 to 31 do Buffer.Append(wBotFace+IntToStr(i)
                                   +BotHeaderEnd+BotSquad[i].Face);
                 for i:=0 to 31 do Buffer.Append(wBotName+IntToStr(i)
                                   +BotHeaderEnd+BotSquad[i].Name);
                 for i:=0 to 31 do Buffer.Append(wBotTeam+IntToStr(i)
                                   +BotHeaderEnd+IntToStr(BotSquad[i].Team));
                 for i:=0 to 31 do Buffer.Append(wBotModel+IntToStr(i)
                                   +BotHeaderEnd+BotSquad[i].ModelClass);
                 for i:=0 to 31 do Buffer.Append(wBotSkin+IntToStr(i)
                                   +BotHeaderEnd+BotSquad[i].Skin);

                 for i:=0 to 31 do Buffer.Append(wBotVoice+IntToStr(i)
                                   +BotHeaderEnd+BotSquad[i].Voice);
                 for i:=0 to 31 do Buffer.Append(wBotSkill+IntToStr(i)
                                   +BotHeaderEnd+SStr(BotSquad[i].Skill));
                 for i:=0 to 31 do Buffer.Append(wBotAccuracy+IntToStr(i)
                                   +BotHeaderEnd+SStr(BotSquad[i].Accuracy));
                 for i:=0 to 31 do Buffer.Append(wBotCombatStyle+IntToStr(i)
                                   +BotHeaderEnd+SStr(BotSquad[i].CombatStyle));
                 for i:=0 to 31 do Buffer.Append(wBotAlertness+IntToStr(i)
                                   +BotHeaderEnd+SStr(BotSquad[i].Alertness));
                 for i:=0 to 31 do Buffer.Append(wBotCamping+IntToStr(i)
                                   +BotHeaderEnd+SStr(BotSquad[i].Camping));
                 for i:=0 to 31 do Buffer.Append(wBotStrafing+IntToStr(i)
                                   +BotHeaderEnd+SStr(BotSquad[i].StrafingAbility));
                 for i:=0 to 31 do Buffer.Append(wBotFave+IntToStr(i)
                                   +BotHeaderEnd+BotSquad[i].FavoriteWeapon);
                 for i:=0 to 31 do if BotSquad[i].Jumpy
                      then Buffer.Append(wBotJumpy+IntToStr(i)+BotHeaderEnd+'1')
                      else Buffer.Append(wBotJumpy+IntToStr(i)+BotHeaderEnd+'0');
               Done:=TRUE;
               Ready:=FALSE;
             end;{else}
    if
       (NOT Ready) {it isn't in the right section} OR
       {it's not a bot}
       (NOT (StartsWith(Uppercase(S),BotName)
            OR StartsWith(Uppercase(S),BotFace)
            OR StartsWith(Uppercase(S),BotSkin)
            OR StartsWith(Uppercase(S),BotModel)
            OR StartsWith(Uppercase(S),BotTeam)
            OR StartsWith(Uppercase(S),BotVoice)
            OR StartsWith(Uppercase(S),BotAccuracy)
            OR StartsWith(Uppercase(S),BotStrafing)
            OR StartsWith(Uppercase(S),BotCamping)
            OR StartsWith(Uppercase(S),BotCombatStyle)
            OR StartsWith(Uppercase(S),BotAlertness)
            OR StartsWith(Uppercase(S),BotFave)
            OR StartsWith(Uppercase(S),BotJumpy)
            OR StartsWith(Uppercase(S),BotSkill)       ))
    then Buffer.Append(S);
  end;{while}
CloseFile(F);

if not Done then ShowMessage('Error: This doesn''t seem to be a UT INI file')
else Buffer.SaveToFile(INIfile);

finally
Buffer.Free;
end;{try}
End;{proc}

function Pick32Bots(BotList:TBigKegOBots):TBotSquad;
var i,j:integer;
    b:TBigKegOBots;
Begin
for i:=0 to 31 do begin
  Result[i]:=BlankBot;
  Result[i].Name:='Bot_'+IntToStr(i);
 end;
b:=TBigKegOBots.Create;
b.init;
try
  if (BotList.Count>0) then begin
    Randomize;
    for i:=0 to BotList.Count-1 do
      b.Append(BotList[i]);
    {to put any default bots at the end, fill the array backwards}
    if (b.Count<32) then i:=b.Count-1 else i:=31;
    repeat
      j:=trunc(Random*b.Count); {pick a random bot, 0 to b.Count-1}
      Result[i]:=b[j];          {put it in the current slot in the array}
      b.Delete(j);              {don't use it again}
      i:=i-1;                   {select the previous slot in the array}
    until (i<0)
   end;{if}
finally
  b.Free;
end;{try}
End;

end.
