{
This file is part of Unreal Tournament System Config version 210f.

Copyright (C) 2000, 2003 Simon McVittie.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
}

UNIT IntLoaderUtils;

INTERFACE

Uses Classes,SysUtils,UTSharedUtils1;

Function SplitTextureLine(const Line:string;var Name,Desc:string):boolean;
Function IntObjectToTSText(intobject:string):string;
Function RemoveObjectHeader(intline:string):string;
Function NonCaseSensitiveValues(T:TStringList;Value:string):string;
Procedure IntObjectToTSLines(intobject:string;var T:TStringList);

IMPLEMENTATION

function RemoveObjectHeader(intline:string):string;
Begin; Result:=Copy(intline,9,length(intline)-9); End;

function NonCaseSensitiveValues(T:TStringList;Value:string):string;
var i:integer;
Begin
Result:='';
if T.Count>0 then begin
  for i:=T.Count-1 downto 0 do
    if StartsWith(Uppercase(T[i]),Uppercase(Value)+'=')
      then Result:=Copy(T[i],Pos('=',T[i])+1,Length(T[i]));
end;
End;


function IntObjectToTSText(intobject:string):string;
var i:integer; Quoted:boolean;
Begin
Quoted:=FALSE;
result:='';
  if (intobject<>'') then
    for i:=1 to length(intobject) do
    begin
      if (NOT Quoted) AND (intobject[i]=',')
      then result:=result+#13#10
      else if intobject[i]='"'
      then Quoted:=NOT Quoted
      else result:=result+intobject[i];
    end;
End;

procedure IntObjectToTSLines(intobject:string;var T:TStringList);
var i:integer; Quoted:boolean; cache:string;
Begin
Quoted:=FALSE;
cache:='';
T.Clear;
  if (intobject<>'') then
    for i:=1 to length(intobject) do
    begin
      if (NOT Quoted) AND (intobject[i]=',')
      then begin T.Append(cache); cache:=''; end
      else if intobject[i]='"'
      then begin Quoted:=NOT Quoted; cache:=cache+'"'; end
      else cache:=cache+intobject[i];
    end;
    T.Append(cache);
End;

Function SplitTextureLine(const Line:string;var Name,Desc:string):boolean;
var T:TStringList; Obj,Cache:string; i:integer; Quoted:boolean;
Begin
T:=TStringList.Create;
Result:=FALSE;
Quoted:=FALSE;
Name:='';
Desc:='';
try
  Obj:=RemoveObjectHeader(Line);
  Cache:='';
  for i:=1 to Length(Obj) do
    begin
      case Obj[i] of
        ',':begin
              if Quoted then Cache:=Cache+Obj[i]
              else begin
                {look at the cache and see what's there}
                if Uppercase(Cache)='CLASS=TEXTURE' then Result:=TRUE
                else if StartsWith(Uppercase(Cache),'NAME=')
                  then Name:=Copy(Cache,6,Length(Cache))
                else if StartsWith(Uppercase(Cache),'DESCRIPTION=')
                  then Desc:=Copy(Cache,13,Length(Cache));
                Cache:='';
              end;
            end;{,}
        '"':begin Cache:=Cache+Obj[i]; Quoted:=NOT Quoted; end;{"}
        else begin Cache:=Cache+Obj[i]; end;{else}
      end;{case}
    end;{for}
    {look at the cache and see what's there}
    if Uppercase(Cache)='CLASS=TEXTURE' then Result:=TRUE
    else if StartsWith(Uppercase(Cache),'NAME=')
      then Name:=Copy(Cache,6,Length(Cache))
    else if StartsWith(Uppercase(Cache),'DESCRIPTION=')
      then Desc:=Copy(Cache,13,Length(Cache));
    Cache:='';
finally
  T.Free;
end;{try}
End;

END.
