{
This file is part of Unreal Tournament System Config version 210f.

Copyright (C) 2000, 2003 Simon McVittie.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
}

unit TModDetailsDefine;

interface

uses Classes;

type TModDetails=class(TObject)
  public
  Name:string;    {mod name}
  Author:string;  {author or team}
  Email:string;   {e-mail address}
  WWW:string;     {URL}
  OtherContact:string; {other contact details}
  Description:string; {about this mod}
  ServerPackages:TStringList; {one per line}
  IsInstalled:boolean;
  TypeHeader:string;
  UtscDatafile:string;
  DatafileCompilerName:string;
  constructor Create;
  destructor Destroy; override;
  function Clone:TModDetails;
  end;{class}

  TModDetailsList=class(TObject)
       private
              PtrList:TList;
       protected
              function GetCount:Integer;
              function GetData(Index:Integer):TModDetails;
              procedure SetData(Index:Integer;R:TModDetails);
       public
              constructor Create;
              destructor Destroy; override;
              procedure Append(R:TModDetails);
              procedure Clear;
              property Count: Integer read GetCount;
              property Mods[Index:Integer]:TModDetails read GetData write SetData;
              

  end;{class}

implementation

constructor TModDetails.Create;
Begin
ServerPackages:=TStringList.Create;
End;

destructor TModDetails.Destroy;
Begin
ServerPackages.Free;
End;

function TModDetails.Clone:TModDetails;
Begin
  Result:=TModDetails.Create;
  Result.Name:=Name;
  Result.Author:=Author;
  Result.Email:=Email;
  Result.WWW:=WWW;
  Result.OtherContact:=OtherContact;
  Result.Description:=Description;
  Result.ServerPackages.CommaText:=Self.ServerPackages.CommaText;
  Result.IsInstalled:=IsInstalled;
  Result.TypeHeader:=TypeHeader;
  Result.UtscDatafile:=UTSCDatafile;
  Result.DatafileCompilerName:=DatafileCompilerName;
End;

constructor TModDetailsList.Create;
Begin
PtrList:=TList.Create;
End;

destructor TModDetailsList.Destroy;
Begin
PtrList.Free;
End;

function TModDetailsList.GetCount:Integer;
Begin
  Result:=PtrList.Count;
End;

function TModDetailsList.GetData(Index:Integer):TModDetails;
var P:^TModDetails;
Begin
  P:=PtrList.Items[Index];
  Result:=P^;
End;

procedure TModDetailsList.SetData(Index:Integer;R:TModDetails);
var Pi:^TModDetails;
Begin
  Pi:=PtrList.Items[Index];
  Pi^:=R;
End;



procedure TModDetailsList.Append(R:TModDetails);
var Pi:^TModDetails;
Begin
  New(Pi);
  Pi^:=R;
  PtrList.Add(Pi);
End;

procedure TModDetailsList.Clear;
var i:integer;
    p:^TModDetails;
Begin
  for i:=0 to PtrList.Count-1 do begin
    p:=PtrList.Items[i];
    if (p<>nil) then begin p^.Free; Dispose(p); end;
    end;
  PtrList.Clear;
End;




end.
