{
This file is part of Unreal Tournament System Config version 210f.

Copyright (C) 2000, 2003 Simon McVittie.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
}

unit ModelIntLoader;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  UTSharedUtils1,TBotModelDefine, StdCtrls, Buttons;

type
  TModelIntLoaderForm = class(TForm)
    Label2: TLabel;
    BitBtn1: TBitBtn;
    Memo1: TMemo;
    BitBtn2: TBitBtn;
    BitBtn3: TBitBtn;
    OpenDialog1: TOpenDialog;
    Label1: TLabel;
    function LoadModelsFromIntFile(IntFile:string):TBotModelList;
  function LoadModels:TBotModelList;
  procedure BitBtn1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    StuffToReturn:TBotModelList;
  end;

var
  ModelIntLoaderForm: TModelIntLoaderForm;

implementation

uses IntLoaderUtils, UTServerConfigGlobal;

{$R *.DFM}

function TModelIntLoaderForm.LoadModelsFromIntFile(IntFile:string):TBotModelList;
var
{  UFile:string;  }
  Buffer,Decoder:TStringList;
  i:integer;
  s:string;
  Model:TBotModel;
  PlayersTemp,BotsTemp:TBotModelList;
Begin
{What to expect in a model's INT:

[Public]
Object=(Name=6Rmko.RumikoBot,Class=Class,MetaClass=BotPack.Bot,Description="Female -Rumiko")
Object=(Name=6Rmko.Rumiko,Class=Class,MetaClass=Botpack.TournamentPlayer,Description="Female -Rumiko")

i) Split off the object= heading.
ii) Split at every comma (probably using tstringlist.commatext).
iii) use tstringlist INI handling techniques to see if it has the bot class.
     if so, check it has the right metaclass.
     if _that_ is true, get description and classnames.
iv) open up the package containing the playerclass and find the declaration
             class <class only of playerclass> extends <whatever>
             if <whatever> contains "tournamentfemale" then it's female
             else it's probably male (nobody's bothered doing extra xan classes)
v) from that position, read on until you reach MESH= and that's the mesh

For future reference, botclass includes ForceMeshToExist which could possibly be
used for class/botclass association.

}
Result:=TBotModelList.Create;
Buffer:=TStringList.Create;
PlayersTemp:=TBotModelList.Create;
BotsTemp:=TBotModelList.Create;
Decoder:=TStringList.Create;
try
  Buffer.LoadFromFile(IntFile);
  if Buffer.Count>0 then for i:=0 to Buffer.Count-1 do
  if StartsWith(UpperCase(Buffer[i]),'OBJECT=(') then
    begin
      Decoder.Text:=IntObjectToTSText(Copy(Buffer[i],9,length(Buffer[i])-9));
      s:=NonCaseSensitiveValues(Decoder,'MetaClass');
      if TextEqu(s,'Botpack.TournamentPlayer') then begin
        Model:=TBotModel.Create;
        Model.Desc:=NonCaseSensitiveValues(Decoder,'Description');
        Model.PlayerClass:=NonCaseSensitiveValues(Decoder,'Name');
        Self.Memo1.Lines.Append('Found a player, class is '
                                         +Model.PlayerClass
                                         +', description is "'
                                         +Model.Desc+'"');
        PlayersTemp.Append(Model);
      end{if}
      else
      if TextEqu(s,'Botpack.Bot') then begin
        Model:=TBotModel.Create;
        Model.Desc:=RemoveQuotes(NonCaseSensitiveValues(Decoder,'Description'));
        Model.BotClass:=NonCaseSensitiveValues(Decoder,'Name');
        Self.Memo1.Lines.Append('Found a bot, class is '
                                         +Model.BotClass
                                         +', description is "'
                                         +Model.Desc+'"');
        BotsTemp.Append(Model);
      end;{if}
    end;{if}
  {Sort out matching - future use}

{@@@ FIXME later}
  for i:=0 to BotsTemp.Count-1 do
    begin

      BotsTemp[i].IsMale:=1;
      BotsTemp[i].HasFaces:=TRUE;
      BotsTemp[i].SkinPackPrefix:='Type mesh name here';
      Result.Append(BotsTemp[i].Clone);
    
    end;
finally
  BotsTemp.Free;
  PlayersTemp.Free;
  Decoder.Free;
  Buffer.Free;
end;{try}
End;

function TModelIntLoaderForm.LoadModels:TBotModelList;
Begin
  Self.Memo1.Clear;
  StuffToReturn.Clear;
  Self.BitBtn3.Enabled:=FALSE;
  if Self.ShowModal=mrOK then begin
    Result:=StuffToReturn.Clone;
  end
  else
    Result:=TBotModelList.Create;
End;

procedure TModelIntLoaderForm.BitBtn1Click(Sender: TObject);
begin
  OpenDialog1.InitialDir:=global.UTBase+'\system';
  Self.Memo1.Clear;
  Self.BitBtn3.Enabled:=FALSE;
  if OpenDialog1.Execute then
    begin
      StuffToReturn.Free;
      StuffToReturn:=LoadModelsFromIntFile(OpenDialog1.Filename);
      BitBtn3.Enabled:=TRUE;
    end;
end;



procedure TModelIntLoaderForm.FormCreate(Sender: TObject);
begin
StuffToReturn:=TBotModelList.Create;

end;

procedure TModelIntLoaderForm.FormDestroy(Sender: TObject);
begin
StuffToReturn.Free;

end;

end.
 