{
This file is part of Unreal Tournament System Config version 210f.

Copyright (C) 2000, 2003 Simon McVittie.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
}

unit SkinINTLoader;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, TBotSkinDefine, IntLoaderUtils, UTSharedUtils1, SvPackFiles;

type
  TSkinIntLoaderForm = class(TForm)
    Label1: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    BitBtn1: TBitBtn;
    Memo1: TMemo;
    BitBtn2: TBitBtn;
    BitBtn3: TBitBtn;
    OpenDialog1: TOpenDialog;
    Label5: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    function LoadAllSkins(Intfile:string):TBotSkinList;
    function LoadSkins:TBotSkinList;
    procedure BitBtn1Click(Sender: TObject);
  private
    { Private declarations }
    Names,Descs:TStringList;
    StuffReturned:TBotSkinList;
  public
    { Public declarations }
  end;

var
  SkinIntLoaderForm: TSkinIntLoaderForm;

implementation

uses UTServerCOnfigGlobal;


{$R *.DFM}

{What to expect in a skin INT:

[Public]
Object=(Name=mymodelSkins_myskin.mysk1,Class=Texture)
Object=(Name=mymodelSkins_myskin.mysk2,Class=Texture,Description="My Skin")
Object=(Name=mymodelSkins_myskin.mysk3Fred,Class=Texture,Description="Fred")
Object=(Name=mymodelSkins_myskin.mysk4,Class=Texture)
Object=(Name=mymodelSkins_myskin.mysk5,Class=Texture)
Object=(Name=mymodelSkins_myskin.mysk5Fred,Class=Texture)

In simplest terms:

i) all entries have package(name) = <packagename>
ii) all classes are Texture
iii) some have descriptions
iv) all have a 4-letter code <code>, then a single-digit number 1~5
-- Log each package/code combination as a new face
v) all 5s are faces and should match one of the others, call it #<face>
vi) all <face>s have <face-specifier> after the number
-- Log each <face-specifier> which has a <desc> as a new face

ALGORITHM
for each line which qualifies, parse into Name, Class, Desc. (3x TSL)
Throw away all lines where Class<>'Texture'. If none left, crash.
Get a packagename.code for each line. (another TSL?)

Repeat:
  Remember the first line in the tables and delete it
  Get all the matching package.code lines as well
  Put them in a further TSL and delete from the original
  In this TSL:
    If there's only one skin element (like in old-style skins) then crash, else:
    Initialise TBotSkin with package.code
    Split TSL by number into 9 TSLs, 1~9, or use iterations to do a virtual split (better).
    Find which number only has one Description where the described option has no
    suffix, this is the skin description.
    Find which one has lots of descriptions, these are the faces. Take suffixes
    from here by removing the numbers, and take descriptions.
    Look for anything with the suffix t_0 and if it's there, assume TCs.
  End;
  If there's some left, go back and try again
}

function TSkinIntLoaderForm.LoadAllSkins(Intfile:string):TBotSkinList;
var Buffer,Decoder:TStringList;
    i,j:integer;
    CurrentPackageAndCode:string;
    NewSkin:TBotSkin;
    xname,xdesc:string;
Begin
Result:=TBotSkinList.Create;
Buffer:=TStringList.Create;
Decoder:=TStringList.Create;
try
  Names.Clear;
  Descs.Clear;
  Buffer.LoadFromFile(Intfile);
  {Memo1.Lines.AddStrings(Buffer);}
  for i:=0 to Buffer.Count-1 do
    begin
      if StartsWith(Uppercase(Buffer[i]),'OBJECT=(')
       then begin
         if SplitTextureLine(Buffer[i],xname,xdesc) then begin
           Names.Append(RemoveQuotes(xname));
           Descs.Append(RemoveQuotes(xdesc));
         end;
       end;
    end;
  {now all the names and descs are in Names and Descs...}
  {B & D no longer needed, so Buffer = current names, Decoder = current descs}
  Buffer.Clear;
  Decoder.Clear;
  while Names.Count>0 do
  begin
    CurrentPackageAndCode:=ExtractPackageName(Names[0]);
    CurrentPackageAndCode:=Copy(Names[0],1,Length(CurrentPackageAndCode)+5);
    {now got the MyMeshSkins_MySkin.mysk bit}
    Buffer.Append(Names[0]);
    Decoder.Append(Descs[0]);
    Names.Delete(0);
    Descs.Delete(0);
    for i:=Names.Count-1 downto 0 do
      if StartsWith(Uppercase(Names[i]),Uppercase(CurrentPackageAndCode)) then
        begin
          Buffer.Append(Names[i]);
          Decoder.Append(Descs[i]);
          Names.Delete(i);
          Descs.Delete(i);
        end;{if}
    {now all of that section are in the buffer and decoder}
    if Buffer.Count<2 then
       begin

       Self.Memo1.Lines.Append('Found old-style skin '+Decoder[0]+
                               '(UT name= '+Buffer[0]+')');
       Memo1.Lines.Append('### This or other skins in this file might not be correct');
       Memo1.Lines.Append('### Please check them after loading');
       NewSkin:=TBotSkin.Create;
      try
        NewSkin.UTName:=Buffer[0];
        NewSkin.Desc:=Decoder[0];
        NewSkin.IsTeamSkinnable:=TRUE;
        Result.Append(NewSkin);
      except
        on Exception do NewSkin.Free;
       end;{try}
      end
    else begin
      NewSkin:=TBotSkin.Create;
      try
        NewSkin.UTName:=CurrentPackageAndCode;
        Self.Memo1.Lines.Append('Found skin '+CurrentPackageAndCode);
        NewSkin.Desc:=CurrentPackageAndCode;
        NewSkin.IsTeamSkinnable:={probably}TRUE;
        for i:=0 to Buffer.Count-1 do
          begin
            for j:=1 to 9 do
              if (Uppercase(Buffer[i])=Uppercase(CurrentPackageAndCode)+IntToStr(j))
                AND (Decoder[i]<>'') then
                begin
                  NewSkin.Desc:=Decoder[i];
                  Self.Memo1.Lines.Append('  -Description is '+NewSkin.Desc);
                end;{if}
            if (Copy(Buffer[i],Length(CurrentPackageAndCode)+2,1)<>'')
                 AND (Decoder[i]<>'')
              then begin
                NewSkin.Faces.Append(ExtractPackageName(CurrentPackageAndCode)+'.'
                                     +Copy(Buffer[i],Length(CurrentPackageAndCode)+2,255)
                                     );
                NewSkin.FaceDescs.Append(Decoder[i]);
                Self.Memo1.Lines.Append('  -Found face '+Decoder[i]+
                                        ' (UT name '+Copy(Buffer[i],Length(CurrentPackageAndCode)+2,255)+')');
              end;
          end;{for}
        Result.Append(NewSkin);
        except
        on Exception do NewSkin.Free;
      end;{try}
    end;{else}
    Buffer.Clear;
    Decoder.Clear;
  end;{while}

finally
Buffer.Free;
Decoder.Free;
Names.Clear;
Descs.Clear;
end;{try}
End;


procedure TSkinIntLoaderForm.FormCreate(Sender: TObject);
begin
Names:=TStringList.Create;
Descs:=TStringList.Create;
StuffReturned:=TBotSkinList.Create;
end;

procedure TSkinIntLoaderForm.FormDestroy(Sender: TObject);
begin
Descs.Free;
Names.Free;
end;

function TSkinIntLoaderForm.LoadSkins;
Begin
  BitBtn1.Enabled:=TRUE;
  Self.Memo1.Clear;
  Self.BitBtn3.Enabled:=FALSE;
  StuffReturned.Clear;
  if Self.ShowModal=mrOK then begin
    Result:=StuffReturned.Clone;
  end
  else
    Result:=TBotSkinList.Create;
End;

procedure TSkinIntLoaderForm.BitBtn1Click(Sender: TObject);
begin
  BitBtn1.Enabled:=FALSE;
  Self.Memo1.Clear;
  Self.BitBtn3.Enabled:=FALSE;
  OpenDialog1.InitialDir:=global.UTBase+'\system';
  if OpenDialog1.Execute then begin
    StuffReturned.Free;
    StuffReturned:=LoadAllSkins(OpenDialog1.Filename);
    BitBtn3.Enabled:=TRUE;
    end;
end;

end.
